#!/bin/bash

#     Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
# 
#     The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
# 
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# 
#    Copyright (c) 2012-2013 by Deon George

# Check our arguments
while getopts ":DF:M:s:v:U" opt; do
	case $opt in
		D) DEBUG=1 ;;

		# Filespace to store in TSM
		F) FILESPACE=$OPTARG ;;

		# Digest to use
		M) DIGEST=$OPTARG ;;

		# Skip VOLs
		s) SKIP_VOL=$OPTARG ;;

		# Only process updates
		U) UPDATE_ONLY=1 ;;

		# Just backup this Vol
		v) VOL=$OPTARG ;;

		# Error Checking
		\?)
			echo "Invalid option: -$OPTARG" >&2
			exit 1
			;;
		:)
			echo "Option -$OPTARG requires an argument." >&2
			exit 1
			;;
	esac
done

# Our defaults
FILESPACE=${FILESPACE:-"/afsdump"}
LASTDB=/var/tmp/tdp.afs.txt

# List our Databases
function list() {
	/usr/sbin/vos listvol localhost -localauth -quiet |awk '{print $1}'
}

# Get Size
function getsize() {
	echo 1024*$(/usr/sbin/vos examine -format -localauth $1|grep diskused | awk '{print $2}')|bc
}

# Last Change
function getlast() {
	/usr/sbin/vos examine -format -localauth $1|grep updateDate | awk '{print $2}'
}

# TSMPIPE ARGUMENTS
[ -n "$DIGEST" ] && TSMPIPE_ARGS="${TSMPIPE_ARGS} -m ${DIGEST}"

# Make sure we have BC
type bc > /dev/null 2>&1
if [ $? -gt 0 ]; then
	echo 'Need BC' && exit 1
fi

list | while read db; do
	[ -n "$VOL" -a "$db" != "$VOL" ] && continue;

	SKIP=0
	if [ -n "$SKIP_VOL" ]; then
		OLDIFS=$IFS
		IFS=":"
		for x in $SKIP_VOL ; do
			[ $x == $db ] && SKIP=1 && break
		done
		IFS=$OLDIFS
	fi

	# We need to skip this VOL
	[ "$SKIP" -eq 1 ] && continue;

	size=$(getsize $db);
	last=$(getlast $db)

	[ -n "$DEBUG" ] && echo "Processing VOL ($db), with size ($size), last updated ($last)"

	# Backup to TSM
	if [ -n "$UPDATE_ONLY" ]; then
		if [ -w $LASTDB ]; then
			last_backup=$(cat $LASTDB|grep -e "\ $db\$" | awk '{print $1}')
			[ -n "$DEBUG" ] && echo "Last backed up ($last_backup)"
			[ $last -eq $last_backup ] && continue
		fi
	fi

	/usr/sbin/vos dump -localauth $db 2>/dev/null| tsmpipe -Bcs ${FILESPACE}/${db} -f FULL -L ${size} ${TSMPIPE_ARGS}
	[ ! -e $LASTDB ] && touch $LASTDB

	if grep -qe "\ ${db}\$" $LASTDB; then
		sed -ie "s/^\([0-9]\+\)\ $db\$/$last $db/" $LASTDB
	else
		echo $last $db >> $LASTDB
	fi
done;
